@@ -78,19 +78,15 @@ class NativeResolverImpl
 	float __get_version_weight(const shared_ptr< const BinaryVersion >&) const;
 	float __get_action_profit(const shared_ptr< const BinaryVersion >&,
 			const shared_ptr< const BinaryVersion >&) const;
-	bool __is_candidate_for_auto_removal(const dg::Element*);
+	AutoRemovalPossibility::Allow __is_candidate_for_auto_removal(const dg::Element*);
 	bool __clean_automatically_installed(Solution&);
 	void __require_strict_relation_expressions();
-	void __pre_apply_action(const Solution&, Solution&, unique_ptr< Action > &&);
+	void __pre_apply_action(const Solution&, Solution&, unique_ptr< Action > &&, size_t);
 	void __calculate_profits(vector< unique_ptr< Action > >& actions) const;
 	void __pre_apply_actions_to_solution_tree(
 			std::function< void (const shared_ptr< Solution >&) > callback,
 			const shared_ptr< Solution >&, vector< unique_ptr< Action > >&);
 
-	void __initial_validate_pass(Solution&);
-	void __validate_element(Solution&, const dg::Element*, size_t);
-	void __validate_changed_package(Solution&, const dg::Element*,
-			const dg::Element*, size_t);
 	void __post_apply_action(Solution&);
 	void __final_verify_solution(const Solution&);
 
