@@ -38,8 +38,8 @@ bool PackageEntry::isModificationAllowed(const dg::Element* elementPtr) const
 	return (findResult == rejectedConflictors.end());
 }
 
-template < class data_t, class Comparator, class KeyGetter >
-class PackageEntryMapBase
+template < class data_t, class KeyGetter >
+class VectorBasedMap
 {
  public:
 	typedef const dg::Element* key_t;
@@ -53,6 +53,13 @@ class PackageEntryMapBase
 	{
 		return static_cast< typename container_t::iterator >(const_cast< iterator_t >(position));
 	}
+	struct __comparator
+	{
+		bool operator()(const data_t& data, const key_t& key) const
+		{
+			return KeyGetter()(data) < key;
+		}
+	};
  public:
 	size_t size() const { return __container.size(); }
 	void reserve(size_t size) { __container.reserve(size); }
@@ -60,11 +67,11 @@ class PackageEntryMapBase
 	const_iterator_t end() const { return &*__container.end(); }
 	const_iterator_t lower_bound(const key_t& key) const
 	{
-		return std::lower_bound(begin(), end(), key, Comparator());
+		return std::lower_bound(begin(), end(), key, __comparator());
 	}
 	iterator_t lower_bound(const key_t& key)
 	{
-		return const_cast< iterator_t >(((const PackageEntryMapBase*)this)->lower_bound(key));
+		return const_cast< iterator_t >(((const VectorBasedMap*)this)->lower_bound(key));
 	}
 	const_iterator_t find(const key_t& key) const
 	{
@@ -90,21 +97,21 @@ class PackageEntryMapBase
 	{
 		__container.push_back(data);
 	}
+	const container_t& getContainer() const
+	{
+		return __container;
+	}
 };
 
-struct PackageEntryMapComparator
-{
-	bool operator()(const pair< const dg::Element*, PackageEntry >& data, const dg::Element* key) const
-	{ return data.first < key; }
-};
+typedef shared_ptr< const PackageEntry > SPPE;
+
 struct PackageEntryMapKeyGetter
 {
-	const dg::Element* operator()(const pair< const dg::Element*, PackageEntry >& data)
+	const dg::Element* operator()(const pair< const dg::Element*, SPPE >& data)
 	{ return data.first; }
 };
-class PackageEntryMap: public PackageEntryMapBase<
-		pair< const dg::Element*, PackageEntry >,
-		PackageEntryMapComparator, PackageEntryMapKeyGetter >
+class PackageEntryMap: public VectorBasedMap<
+		pair< const dg::Element*, SPPE >, PackageEntryMapKeyGetter >
 {
  public:
 	mutable size_t forkedCount;
@@ -114,31 +121,39 @@ class PackageEntryMap: public PackageEntryMapBase<
 	{}
 };
 
-
-struct PackageEntrySetComparator
+struct BrokenSuccessorMapKeyGetter
 {
-	bool operator()(const dg::Element* data, const dg::Element* key) const
-	{ return data < key; }
+	const dg::Element* operator()(const BrokenSuccessor& data) { return data.elementPtr; }
 };
-struct PackageEntrySetKeyGetter
-{
-	const dg::Element* operator()(const dg::Element* data) { return data; }
-};
-class PackageEntrySet: public PackageEntryMapBase< const dg::Element*,
-		PackageEntrySetComparator, PackageEntrySetKeyGetter >
+class BrokenSuccessorMap: public VectorBasedMap< BrokenSuccessor, BrokenSuccessorMapKeyGetter >
 {};
 
+SolutionStorage::Change::Change(size_t parentSolutionId_)
+	: parentSolutionId(parentSolutionId_)
+{}
 
 SolutionStorage::SolutionStorage(const Config& config, const Cache& cache)
 	: __next_free_id(1), __dependency_graph(config, cache)
 {}
 
+size_t SolutionStorage::__get_new_solution_id(const Solution& parent)
+{
+	__change_index.emplace_back(parent.id);
+	return __next_free_id++;
+}
+
+shared_ptr< Solution > SolutionStorage::fakeCloneSolution(const shared_ptr< Solution >& source)
+{
+	source->id = __get_new_solution_id(*source);
+	return source;
+}
+
 shared_ptr< Solution > SolutionStorage::cloneSolution(const shared_ptr< Solution >& source)
 {
-	shared_ptr< Solution > cloned(new Solution);
+	auto cloned = std::make_shared< Solution >();
 	cloned->score = source->score;
 	cloned->level = source->level;
-	cloned->id = __next_free_id++;
+	cloned->id = __get_new_solution_id(*source);
 	cloned->finished = false;
 
 	cloned->__parent = source;
@@ -199,11 +214,12 @@ bool SolutionStorage::simulateSetPackageEntry(const Solution& solution,
 	return true;
 }
 
-void SolutionStorage::setRejection(Solution& solution, const dg::Element* elementPtr)
+void SolutionStorage::setRejection(Solution& solution,
+		const dg::Element* elementPtr, const dg::Element* dontChangePtr)
 {
 	const dg::Element* conflictingElementPtr;
 	simulateSetPackageEntry(solution, elementPtr, &conflictingElementPtr);
-	if (!conflictingElementPtr)
+	if (!conflictingElementPtr || conflictingElementPtr == dontChangePtr)
 	{
 		return;
 	}
@@ -214,43 +230,153 @@ void SolutionStorage::setRejection(Solution& solution, const dg::Element* elemen
 
 	packageEntry.rejectedConflictors.push_front(elementPtr);
 	setPackageEntry(solution, conflictingElementPtr,
-			std::move(packageEntry), NULL);
+			std::move(packageEntry), NULL, -1);
+}
+
+void SolutionStorage::__update_broken_successors(Solution& solution,
+		const dg::Element* oldElementPtr, const dg::Element* newElementPtr, size_t priority)
+{
+	if (priority == (size_t)-1)
+	{
+		return;
+	}
+
+	auto& bss = *solution.__broken_successors;
+
+	auto reverseDependencyExists = [this, &solution](const dg::Element* elementPtr)
+	{
+		for (auto reverseDependencyPtr: getPredecessorElements(elementPtr))
+		{
+			if (solution.getPackageEntry(reverseDependencyPtr))
+			{
+				return true;
+			}
+		}
+		return false;
+	};
+	auto isPresent = [](const GraphCessorListType& container, const dg::Element* elementPtr)
+	{
+		return std::find(container.begin(), container.end(), elementPtr) != container.end();
+	};
+
+	static const GraphCessorListType nullList;
+
+	const auto& successorsOfOld = oldElementPtr ? getSuccessorElements(oldElementPtr) : nullList;
+	const auto& successorsOfNew = getSuccessorElements(newElementPtr);
+	// check direct dependencies of the old element
+	for (auto successorPtr: successorsOfOld)
+	{
+		if (isPresent(successorsOfNew, successorPtr)) continue;
+
+		auto it = bss.find(successorPtr);
+		if (it != bss.end())
+		{
+			if (!reverseDependencyExists(successorPtr))
+			{
+				bss.erase(it);
+			}
+		}
+	}
+	// check direct dependencies of the new element
+	for (auto successorPtr: successorsOfNew)
+	{
+		if (isPresent(successorsOfOld, successorPtr)) continue;
+
+		auto it = bss.lower_bound(successorPtr);
+		if (it == bss.end() || it->elementPtr != successorPtr)
+		{
+			if (!verifyElement(solution, successorPtr))
+			{
+				bss.insert(it, BrokenSuccessor { successorPtr, priority });
+			}
+		}
+		else
+		{
+			it->priority = std::max(it->priority, priority);
+		}
+	}
+
+	const auto& predecessorsOfOld = oldElementPtr ? getPredecessorElements(oldElementPtr) : nullList;
+	const auto& predecessorsOfNew = getPredecessorElements(newElementPtr);
+	// invalidate those which depend on the old element
+	for (auto predecessorElementPtr: predecessorsOfOld)
+	{
+		if (isPresent(predecessorsOfNew, predecessorElementPtr)) continue;
+
+		if (reverseDependencyExists(predecessorElementPtr))
+		{
+			if (!verifyElement(solution, predecessorElementPtr))
+			{
+				// here we assume brokenSuccessors didn't
+				// contain predecessorElementPtr, since as old element was
+				// present, predecessorElementPtr was not broken
+				bss.insert(bss.lower_bound(predecessorElementPtr),
+						BrokenSuccessor { predecessorElementPtr, priority });
+			}
+		}
+	}
+	// validate those which depend on the new element
+	for (auto predecessorElementPtr: predecessorsOfNew)
+	{
+		if (isPresent(predecessorsOfOld, predecessorElementPtr)) continue;
+
+		auto it = bss.find(predecessorElementPtr);
+		if (it != bss.end())
+		{
+			bss.erase(it);
+		}
+	}
+}
+
+void SolutionStorage::__update_change_index(size_t solutionId,
+		const dg::Element* newElementPtr, const PackageEntry& packageEntry)
+{
+	if (!packageEntry.sticked)
+	{
+		return; // not a "main" change
+	}
+
+	__change_index[solutionId].insertedElementPtr = newElementPtr;
 }
 
 void SolutionStorage::setPackageEntry(Solution& solution,
 		const dg::Element* elementPtr, PackageEntry&& packageEntry,
-		const dg::Element* conflictingElementPtr)
+		const dg::Element* conflictingElementPtr, size_t priority)
 {
 	__dependency_graph.unfoldElement(elementPtr);
+	__update_change_index(solution.id, elementPtr, packageEntry);
 
 	auto it = solution.__added_entries->lower_bound(elementPtr);
 	if (it == solution.__added_entries->end() || it->first != elementPtr)
 	{
 		// there is no modifiable element in this solution
-		solution.__added_entries->insert(it, make_pair(elementPtr, std::move(packageEntry)));
+		solution.__added_entries->insert(it,
+				make_pair(elementPtr, std::make_shared< const PackageEntry >(std::move(packageEntry))));
 
 		if (conflictingElementPtr)
 		{
-			auto forRemovalIt = solution.__added_entries->find(conflictingElementPtr);
-			if (forRemovalIt != solution.__added_entries->end())
+			auto forRemovalIt = solution.__added_entries->lower_bound(conflictingElementPtr);
+			if (forRemovalIt != solution.__added_entries->end() && forRemovalIt->first == conflictingElementPtr)
 			{
-				solution.__added_entries->erase(forRemovalIt);
+				forRemovalIt->second.reset();
+			}
+			else
+			{
+				solution.__added_entries->insert(forRemovalIt, { conflictingElementPtr, {} });
 			}
-			// may be present in master too, act safe
-			solution.__removed_entries->insert(
-					solution.__removed_entries->lower_bound(conflictingElementPtr),
-					(const dg::Element*)conflictingElementPtr);
 		}
 	}
 	else
 	{
 		if (conflictingElementPtr)
 		{
-			fatal2("internal error: conflicting elements in __added_entries: solution '%u', in '%s', out '%s'",
+			fatal2i("conflicting elements in __added_entries: solution '%u', in '%s', out '%s'",
 					solution.id, elementPtr->toString(), conflictingElementPtr->toString());
 		}
-		it->second = std::move(packageEntry);
+		it->second = std::make_shared< const PackageEntry >(std::move(packageEntry));
 	}
+
+	__update_broken_successors(solution, conflictingElementPtr, elementPtr, priority);
 }
 
 void SolutionStorage::prepareForResolving(Solution& initialSolution,
@@ -259,8 +385,8 @@ void SolutionStorage::prepareForResolving(Solution& initialSolution,
 {
 	auto source = __dependency_graph.fill(oldPackages, initialPackages);
 
-	auto comparator = [](const pair< const dg::Element*, PackageEntry >& left,
-			const pair< const dg::Element*, PackageEntry >& right)
+	auto comparator = [](const pair< const dg::Element*, SPPE >& left,
+			const pair< const dg::Element*, SPPE >& right)
 	{
 		return left.first < right.first;
 	};
@@ -272,6 +398,12 @@ void SolutionStorage::prepareForResolving(Solution& initialSolution,
 		__dependency_graph.unfoldElement(it->first);
 		initialSolution.__added_entries->push_back(*it);
 	}
+	for (const auto& entry: *initialSolution.__added_entries)
+	{
+		__update_broken_successors(initialSolution, NULL, entry.first, 0);
+	}
+
+	__change_index.emplace_back(0);
 }
 
 bool SolutionStorage::verifyElement(const Solution& solution,
@@ -316,26 +448,77 @@ void SolutionStorage::unfoldElement(const dg::Element* elementPtr)
 	__dependency_graph.unfoldElement(elementPtr);
 }
 
+vector< const dg::Element* > SolutionStorage::getInsertedElements(const Solution& solution) const
+{
+	vector< const dg::Element* > result;
+
+	auto solutionId = solution.id;
+	while (solutionId != 0)
+	{
+		const auto& change = __change_index[solutionId];
+		result.push_back(change.insertedElementPtr);
+		solutionId = change.parentSolutionId;
+	}
+	std::reverse(result.begin(), result.end());
+	return result;
+}
 
 Solution::Solution()
 	: id(0), level(0), finished(false), score(0)
 {
 	__added_entries.reset(new PackageEntryMap);
-	__removed_entries.reset(new PackageEntrySet);
+	__broken_successors = new BrokenSuccessorMap;
+}
+
+Solution::~Solution()
+{
+	delete __broken_successors;
+}
+
+template < typename CallbackType >
+void __foreach_solution_element(const PackageEntryMap& masterEntries, const PackageEntryMap& addedEntries,
+		CallbackType callback)
+{
+	class RepackInsertIterator: public std::iterator< std::output_iterator_tag, PackageEntryMap::value_type >
+	{
+		const CallbackType& __callback;
+	 public:
+		RepackInsertIterator(const CallbackType& callback_)
+			: __callback(callback_) {}
+		RepackInsertIterator& operator++() { return *this; }
+		RepackInsertIterator& operator*() { return *this; }
+		void operator=(const PackageEntryMap::value_type& data)
+		{
+			if (data.second)
+			{
+				__callback(data);
+			}
+		}
+	};
+	struct Comparator
+	{
+		bool operator()(const PackageEntryMap::value_type& left, const PackageEntryMap::value_type& right)
+		{ return left.first < right.first; }
+	};
+	// it's important that parent's __added_entries come first,
+	// if two elements are present in both (i.e. an element is overriden)
+	// the new version of an element will be considered
+	std::set_union(addedEntries.begin(), addedEntries.end(),
+			masterEntries.begin(), masterEntries.end(),
+			RepackInsertIterator(callback), Comparator());
 }
 
 void Solution::prepare()
 {
 	if (!__parent)
 	{
-		fatal2("internal error: undefined master solution");
+		return; // prepared already
 	}
 
 	if (!__parent->__master_entries)
 	{
 		// parent solution is a master solution, build a slave on top of it
 		__master_entries = __parent->__added_entries;
-		*__removed_entries = (*__parent->__removed_entries);
 	}
 	else
 	{
@@ -350,46 +533,18 @@ void Solution::prepare()
 			__added_entries->reserve(__parent->__added_entries->size() +
 					__parent->__master_entries->size());
 
-			class RepackInsertIterator: public std::iterator< std::output_iterator_tag, PackageEntryMap::value_type >
-			{
-				PackageEntryMap& __target;
-				PackageEntrySet& __exceptions;
-			 public:
-				RepackInsertIterator(PackageEntryMap& target, PackageEntrySet& exceptions)
-					: __target(target), __exceptions(exceptions) {}
-				RepackInsertIterator& operator++() { return *this; }
-				RepackInsertIterator& operator*() { return *this; }
-				void operator=(const PackageEntryMap::value_type& data)
-				{
-					auto presentIt = __exceptions.find(data.first);
-					if (presentIt == __exceptions.end())
-					{
-						__target.push_back(data);
-					}
-				}
-			};
-			struct Comparator
-			{
-				bool operator()(const PackageEntryMap::value_type& left, const PackageEntryMap::value_type& right)
-				{ return left.first < right.first; }
-			};
-			// it's important that parent's __added_entries come first,
-			// if two elements are present in both (i.e. an element is overriden)
-			// the new version of an element will be written
-			std::set_union(__parent->__added_entries->begin(), __parent->__added_entries->end(),
-					__parent->__master_entries->begin(), __parent->__master_entries->end(),
-					RepackInsertIterator(*__added_entries, *__parent->__removed_entries), Comparator());
+			__foreach_solution_element(*__parent->__master_entries, *__parent->__added_entries,
+					[this](const PackageEntryMap::value_type& data) { this->__added_entries->push_back(data); });
 		}
 		else
 		{
 			// build new slave solution from current
 			__master_entries = __parent->__master_entries;
 			*__added_entries = *(__parent->__added_entries);
-			*__removed_entries = *(__parent->__removed_entries);
 		}
 	}
 
-	insertedElementPtrs = __parent->insertedElementPtrs;
+	__broken_successors = new BrokenSuccessorMap(*__parent->__broken_successors);
 	__parent.reset();
 }
 
@@ -397,103 +552,18 @@ vector< const dg::Element* > Solution::getElements() const
 {
 	vector< const dg::Element* > result;
 
-	if (__master_entries)
-	{
-		FORIT(it, *__master_entries)
-		{
-			if (__removed_entries->find(it->first) == __removed_entries->end())
-			{
-				result.push_back(it->first);
-			}
-		}
-	}
-	auto middleSize = result.size();
-	FORIT(it, *__added_entries)
-	{
-		result.push_back(it->first);
-	}
+	static const PackageEntryMap nullPackageEntryMap;
+	const auto& masterEntries = __master_entries ? *__master_entries : nullPackageEntryMap;
 
-	std::inplace_merge(result.begin(), result.begin() + middleSize, result.end());
-	result.erase(std::unique(result.begin(), result.end()), result.end());
+	__foreach_solution_element(masterEntries, *__added_entries,
+			[&result](const PackageEntryMap::value_type& data) { result.push_back(data.first); });
 
 	return result;
 }
 
-void Solution::getBrokenPairs(const std::function< void (BrokenPairType&&) >& callback) const
+const vector< BrokenSuccessor >& Solution::getBrokenSuccessors() const
 {
-	auto isEligible = [](PackageEntryMap::const_iterator_t it) -> bool
-	{
-		return !it->second.brokenSuccessors.empty();
-	};
-	auto processEntry = [this, &isEligible, &callback](PackageEntryMap::const_iterator_t it)
-	{
-		if (__removed_entries->find(it->first) == __removed_entries->end())
-		{
-			FORIT(brokenSuccessorIt, it->second.brokenSuccessors)
-			{
-				callback(make_pair(it->first, *brokenSuccessorIt));
-			}
-		}
-	};
-
-	auto masterIt = __master_entries ? __master_entries->begin() : __added_entries->end();
-	auto masterEnd = __master_entries ? __master_entries->end() : __added_entries->end();
-	auto ownIt = __added_entries->begin();
-	auto ownEnd = __added_entries->end();
-
-	--masterIt;
-	--ownIt;
-	{
-		goto both;
-	compare:
-		// speeding up a bit, usually most of masterIt won't be included anyway
-		if (!isEligible(masterIt))
-		{
-			goto master;
-		}
-		if (masterIt->first < ownIt->first)
-		{
-			processEntry(masterIt);
-			goto master;
-		}
-		else if (masterIt->first > ownIt->first)
-		{
-			processEntry(ownIt);
-			goto own;
-		}
-		else // equal keys
-		{
-			// own entry overrides master entry
-			processEntry(ownIt);
-			goto both;
-		}
-	both:
-		++masterIt;
-		++ownIt;
-		if (masterIt != masterEnd)
-		{
-			if (ownIt != ownEnd) goto compare; else goto only_master;
-		}
-		else
-		{
-			if (ownIt != ownEnd) goto only_own; else goto end;
-		}
-	master:
-		++masterIt;
-		if (masterIt != masterEnd) goto compare; else goto only_own;
-	own:
-		++ownIt;
-		if (ownIt != ownEnd) goto compare; else goto only_master;
-	only_master:
-		processEntry(masterIt);
-		++masterIt;
-		if (masterIt != masterEnd) goto only_master; else goto end;
-	only_own:
-		processEntry(ownIt);
-		++ownIt;
-		if (ownIt != ownEnd) goto only_own; else goto end;
-	end:;
-	}
+	return __broken_successors->getContainer();
 }
 
 const PackageEntry* Solution::getPackageEntry(const dg::Element* elementPtr) const
@@ -501,18 +571,14 @@ const PackageEntry* Solution::getPackageEntry(const dg::Element* elementPtr) con
 	auto it = __added_entries->find(elementPtr);
 	if (it != __added_entries->end())
 	{
-		return &it->second;
+		return it->second.get();
 	}
 	if (__master_entries)
 	{
 		it = __master_entries->find(elementPtr);
 		if (it != __master_entries->end())
 		{
-			if (__removed_entries->find(elementPtr) != __removed_entries->end())
-			{
-				return NULL;
-			}
-			return &it->second;
+			return it->second.get();
 		}
 	}
 
