@@ -58,21 +58,9 @@ struct PackageEntry
 			return brokenElementPtr->getReason(*versionElementPtr);
 		}
 	};
-	struct BrokenSuccessor
-	{
-		const dg::Element* elementPtr;
-		size_t priority;
-
-		BrokenSuccessor()
-		{}
-		BrokenSuccessor(const dg::Element* elementPtr_, size_t priority_)
-			: elementPtr(elementPtr_), priority(priority_)
-		{}
-	};
 
 	bool sticked;
 	bool autoremoved;
-	forward_list< BrokenSuccessor > brokenSuccessors;
 	forward_list< const dg::Element* > rejectedConflictors;
 	IntroducedBy introducedBy;
 
@@ -87,7 +75,13 @@ struct PackageEntry
 };
 
 class PackageEntryMap;
-class PackageEntrySet;
+class BrokenSuccessorMap;
+
+struct BrokenSuccessor
+{
+	const dg::Element* elementPtr;
+	size_t priority;
+};
 
 class Solution
 {
@@ -96,24 +90,23 @@ class Solution
 	shared_ptr< const Solution > __parent;
 	shared_ptr< const PackageEntryMap > __master_entries;
 	shared_ptr< PackageEntryMap > __added_entries;
-	shared_ptr< PackageEntrySet > __removed_entries;
+	BrokenSuccessorMap*  __broken_successors;
  public:
 	size_t id;
 	size_t level;
 	bool finished;
 	ssize_t score;
 	std::unique_ptr< const void > pendingAction;
-	vector< const dg::Element* > insertedElementPtrs; // in time order
 
 	Solution();
 	Solution(const Solution&) = delete;
 	Solution& operator=(const Solution&) = delete;
+	~Solution();
 
 	void prepare();
 	vector< const dg::Element* > getElements() const;
 
-	typedef pair< const dg::Element*, PackageEntry::BrokenSuccessor > BrokenPairType;
-	void getBrokenPairs(const std::function< void (BrokenPairType&&) >&) const;
+	const vector< BrokenSuccessor >& getBrokenSuccessors() const;
 	// result becomes invalid after any setPackageEntry
 	const PackageEntry* getPackageEntry(const dg::Element*) const;
 };
@@ -121,10 +114,28 @@ class Solution
 class SolutionStorage
 {
 	size_t __next_free_id;
+	size_t __get_new_solution_id(const Solution& parent);
+
 	dg::DependencyGraph __dependency_graph;
+
+	void __update_broken_successors(Solution&,
+			const dg::Element*, const dg::Element*, size_t priority);
+
+	struct Change
+	{
+		const dg::Element* insertedElementPtr;
+		size_t parentSolutionId;
+
+		explicit Change(size_t);
+	};
+	vector< Change > __change_index;
+	void __update_change_index(size_t, const dg::Element*, const PackageEntry&);
  public:
 	SolutionStorage(const Config&, const Cache& cache);
+
 	shared_ptr< Solution > cloneSolution(const shared_ptr< Solution >&);
+	shared_ptr< Solution > fakeCloneSolution(const shared_ptr< Solution >&);
+
 	void prepareForResolving(Solution&,
 			const map< string, shared_ptr< const BinaryVersion > >&,
 			const map< string, dg::InitialPackageEntry >&);
@@ -138,10 +149,12 @@ class SolutionStorage
 			getConflictingElements(const dg::Element*);
 	bool simulateSetPackageEntry(const Solution& solution,
 			const dg::Element*, const dg::Element**) const;
-	void setRejection(Solution&, const dg::Element*);
+	void setRejection(Solution&, const dg::Element*, const dg::Element*);
 	void setPackageEntry(Solution&, const dg::Element*,
-			PackageEntry&&, const dg::Element*);
+			PackageEntry&&, const dg::Element*, size_t);
 	void unfoldElement(const dg::Element*);
+
+	vector< const dg::Element* > getInsertedElements(const Solution& solution) const;
 };
 
 }
